function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

export var PARSER_ALLOW = 1;
export var PARSER_DENY = 2;
export var TYPE_INLINE = 'inline';
export var TYPE_BLOCK = 'block';
export var TYPE_INLINE_BLOCK = 'inline-block';
export var CONFIG_INLINE = {
  rule: PARSER_ALLOW,
  type: TYPE_INLINE,
  inline: true,
  block: false,
  self: false,
  void: false,
  parent: [],
  children: []
};
export var CONFIG_BLOCK = {
  rule: PARSER_ALLOW,
  type: TYPE_BLOCK,
  inline: true,
  block: true,
  self: true,
  void: false,
  parent: [],
  children: []
};
var tagConfigs = {
  a: {
    type: TYPE_INLINE_BLOCK,
    block: true
  },
  address: {
    self: false
  },
  audio: {
    children: ['track', 'source']
  },
  br: {
    inline: false,
    void: true
  },
  button: {
    type: TYPE_INLINE_BLOCK
  },
  dd: {
    parent: ['dl']
  },
  dl: {
    children: ['dt']
  },
  dt: {
    parent: ['dl'],
    children: ['dd']
  },
  figcaption: {
    parent: ['figure']
  },
  footer: {
    self: false
  },
  header: {
    self: false
  },
  h1: {
    self: false
  },
  h2: {
    self: false
  },
  h3: {
    self: false
  },
  h4: {
    self: false
  },
  h5: {
    self: false
  },
  h6: {
    self: false
  },
  hr: {
    inline: false,
    block: false,
    void: true
  },
  img: {
    inline: false,
    void: true
  },
  li: {
    self: false,
    parent: ['ul', 'ol']
  },
  main: {
    self: false
  },
  ol: {
    children: ['li']
  },
  picture: {
    children: ['source', 'img']
  },
  source: {
    inline: false,
    parent: ['audio', 'video', 'picture'],
    void: true
  },
  span: {
    self: true
  },
  table: {
    children: ['thead', 'tbody', 'tfoot', 'tr']
  },
  tbody: {
    parent: ['table'],
    children: ['tr']
  },
  td: {
    parent: ['tr']
  },
  tfoot: {
    parent: ['table'],
    children: ['tr']
  },
  th: {
    parent: ['tr']
  },
  thead: {
    parent: ['table'],
    children: ['tr']
  },
  tr: {
    parent: ['table', 'tbody', 'thead', 'tfoot'],
    children: ['th', 'td']
  },
  track: {
    inline: false,
    parent: ['audio', 'video'],
    void: true
  },
  ul: {
    children: ['li']
  },
  video: {
    children: ['track', 'source']
  }
};

function createConfigBuilder(config) {
  return function (tagName) {
    tagConfigs[tagName] = _extends({}, config, tagConfigs[tagName]);
  };
}

['a', 'abbr', 'b', 'br', 'button', 'canvas', 'cite', 'code', 'del', 'dfn', 'em', 'i', 'iframe', 'img', 'ins', 'kbd', 'label', 'mark', 'output', 'picture', 'q', 's', 'samp', 'script', 'source', 'span', 'strong', 'sub', 'sup', 'style', 'time', 'track', 'u', 'var', 'video'].forEach(createConfigBuilder(CONFIG_INLINE));
['address', 'article', 'aside', 'audio', 'blockquote', 'dd', 'details', 'div', 'dl', 'dt', 'fieldset', 'figcaption', 'figure', 'footer', 'header', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'legend', 'li', 'main', 'nav', 'ol', 'p', 'pre', 'section', 'summary', 'table', 'thead', 'tbody', 'tfoot', 'tr', 'th', 'td', 'ul'].forEach(createConfigBuilder(CONFIG_BLOCK));
export var TAGS = Object.freeze(tagConfigs);
export var BANNED_TAG_LIST = ['applet', 'body', 'canvas', 'embed', 'frame', 'frameset', 'head', 'html', 'iframe', 'object', 'script', 'style'];
export var ALLOWED_TAG_LIST = Object.keys(TAGS);
export var FILTER_ALLOW = 1;
export var FILTER_DENY = 2;
export var FILTER_CAST_NUMBER = 3;
export var FILTER_CAST_BOOL = 4;
export var ATTRIBUTES = Object.freeze({
  alt: FILTER_ALLOW,
  cite: FILTER_ALLOW,
  class: FILTER_ALLOW,
  colspan: FILTER_CAST_NUMBER,
  controls: FILTER_CAST_BOOL,
  datetime: FILTER_ALLOW,
  default: FILTER_CAST_BOOL,
  disabled: FILTER_CAST_BOOL,
  dir: FILTER_ALLOW,
  height: FILTER_ALLOW,
  href: FILTER_ALLOW,
  id: FILTER_ALLOW,
  kind: FILTER_ALLOW,
  label: FILTER_ALLOW,
  lang: FILTER_ALLOW,
  loop: FILTER_CAST_BOOL,
  muted: FILTER_CAST_BOOL,
  poster: FILTER_ALLOW,
  role: FILTER_ALLOW,
  rowspan: FILTER_CAST_NUMBER,
  sizes: FILTER_ALLOW,
  span: FILTER_CAST_NUMBER,
  src: FILTER_ALLOW,
  srcset: FILTER_ALLOW,
  target: FILTER_ALLOW,
  title: FILTER_ALLOW,
  type: FILTER_ALLOW,
  width: FILTER_ALLOW
});
export var ATTRIBUTES_TO_PROPS = Object.freeze({
  class: 'className',
  colspan: 'colSpan',
  datetime: 'dateTime',
  rowspan: 'rowSpan',
  srcset: 'srcSet'
});